@@ -1,16 +1,16 @@
-from goyabucli.scraper import Anime,Scraper
+from .scraper import Anime,Scraper
 from typing import List,Union
-from datetime import timezone
-from datetime import datetime
+from datetime import datetime, timezone
 from os import path, makedirs
 import json
-from goyabucli.utils import getTotalEpisodesCount
-from goyabucli.dropdown import interactiveTable,bcolors
-from goyabucli.translation import t
+from .utils import getTotalEpisodesCount
+from .dropdown import interactiveTable,bcolors
+from .translation import t
+from concurrent.futures import ThreadPoolExecutor, as_completed
 
 
 class SessionItem():
-    def __init__(self, anime:Anime, date_utc:int, episodesInTotal:int, availableEpisodes:int, lastEpisode:int, lastSource:str, watchTime=0):
+    def __init__(self, anime:Anime, date_utc:int, episodesInTotal:int, availableEpisodes:int, lastEpisode:int, lastSource:str, watchTime=0, duration=0):
         self.anime = anime
 
         self.date_utc = datetime.fromtimestamp(date_utc,timezone.utc)
@@ -19,6 +19,7 @@ class SessionItem():
         self.watchTime = watchTime
         self.lastEpisode = lastEpisode
         self.lastSource = lastSource
+        self.duration = duration
 
     @property
     def id(self):
@@ -87,7 +88,7 @@ class SessionManager():
                     )
                 )
 
-    def update(self, anime:Anime, lastEpisode:int, watchTime=0):
+    def update(self, anime:Anime, lastEpisode:int, watchTime=0, duration=0):
         right_sessionItem = next(item for item in self.session_items if item.id == anime.id)
 
         if not right_sessionItem:
@@ -95,6 +96,7 @@ class SessionManager():
 
         right_sessionItem.lastEpisode = lastEpisode
         right_sessionItem.watchTime = watchTime
+        right_sessionItem.duration = duration
 
     def remove(self, id:str):
         right_sessionItem = next(item for item in self.session_items if item.id == id)
@@ -118,7 +120,8 @@ class SessionManager():
                 json_anime['availableEpisodes'],
                 json_anime['lastEpisode'],
                 json_anime['lastSource'],
-                watchTime=json_anime['watchTime']
+                watchTime=json_anime['watchTime'],
+                duration=json_anime['duration'],
             ))
         
         return session_items
@@ -126,15 +129,7 @@ class SessionManager():
     def dump(self, verbose=False, number_to_update=0):
         content = {}
 
-        pbar = None
-        if verbose:
-            from tqdm import tqdm
-            pbar = tqdm(total=len(self.session_items), postfix=t('Animes atualizados'), ascii=True, leave=False, bar_format='|{bar}| {n_fmt}/{total_fmt}{postfix}')
-
-        for i,session_item in enumerate(self.session_items):
-            if pbar:
-                pbar.update(1)
-
+        def updateSessionItem(i, session_item):
             availableEpisodes = session_item.availableEpisodes
             if number_to_update and i+1>len(self.session_items)-number_to_update:
                 availableEpisodes = len(session_item.anime.retrieveEpisodes())
@@ -147,9 +142,21 @@ class SessionManager():
                 'availableEpisodes': availableEpisodes,
                 'lastEpisode': session_item.lastEpisode,
                 'lastSource': session_item.lastSource,
-                'watchTime': session_item.watchTime
+                'watchTime': session_item.watchTime,
+                'duration': session_item.duration
             }
 
+        pbar = None
+        if verbose:
+            from tqdm import tqdm
+            pbar = tqdm(total=len(self.session_items), postfix=t('Animes atualizados'), ascii=True, leave=False, bar_format='|{bar}| {n_fmt}/{total_fmt}{postfix}')
+
+        with ThreadPoolExecutor(max_workers=4) as executor:
+            futures = [executor.submit(updateSessionItem,i, session_item) for i,session_item in enumerate(self.session_items)]
+            for _ in as_completed(futures):
+                if pbar:
+                    pbar.update(1)
+
         if pbar:
             pbar.close()
 
